@@ -21,7 +21,7 @@ use crate::{
     admin::{
         call_context::CallContext,
         model::{CompatSession, Resource},
-        params::Pagination,
+        params::{IncludeCount, Pagination},
         response::{ErrorResponse, PaginatedResponse},
     },
     impl_from_error_for_route,
@@ -137,16 +137,22 @@ Use the `filter[status]` parameter to filter the sessions by their status and `p
             let sessions = CompatSession::samples();
             let pagination = mas_storage::Pagination::first(sessions.len());
             let page = Page {
-                edges: sessions.into(),
+                edges: sessions
+                    .into_iter()
+                    .map(|node| mas_storage::pagination::Edge {
+                        cursor: node.id(),
+                        node,
+                    })
+                    .collect(),
                 has_next_page: true,
                 has_previous_page: false,
             };
 
             t.description("Paginated response of compatibility sessions")
-                .example(PaginatedResponse::new(
+                .example(PaginatedResponse::for_page(
                     page,
                     pagination,
-                    42,
+                    Some(42),
                     CompatSession::PATH,
                 ))
         })
@@ -159,10 +165,11 @@ Use the `filter[status]` parameter to filter the sessions by their status and `p
 #[tracing::instrument(name = "handler.admin.v1.compat_sessions.list", skip_all)]
 pub async fn handler(
     CallContext { mut repo, .. }: CallContext,
-    Pagination(pagination): Pagination,
+    Pagination(pagination, include_count): Pagination,
     params: FilterParams,
 ) -> Result<Json<PaginatedResponse<CompatSession>>, RouteError> {
     let base = format!("{path}{params}", path = CompatSession::PATH);
+    let base = include_count.add_to_base(&base);
     let filter = CompatSessionFilter::default();
 
     // Load the user from the filter
@@ -206,15 +213,31 @@ pub async fn handler(
         None => filter,
     };
 
-    let page = repo.compat_session().list(filter, pagination).await?;
-    let count = repo.compat_session().count(filter).await?;
+    let response = match include_count {
+        IncludeCount::True => {
+            let page = repo
+                .compat_session()
+                .list(filter, pagination)
+                .await?
+                .map(CompatSession::from);
+            let count = repo.compat_session().count(filter).await?;
+            PaginatedResponse::for_page(page, pagination, Some(count), &base)
+        }
+        IncludeCount::False => {
+            let page = repo
+                .compat_session()
+                .list(filter, pagination)
+                .await?
+                .map(CompatSession::from);
+            PaginatedResponse::for_page(page, pagination, None, &base)
+        }
+        IncludeCount::Only => {
+            let count = repo.compat_session().count(filter).await?;
+            PaginatedResponse::for_count_only(count, &base)
+        }
+    };
 
-    Ok(Json(PaginatedResponse::new(
-        page.map(CompatSession::from),
-        pagination,
-        count,
-        &base,
-    )))
+    Ok(Json(response))
 }
 
 #[cfg(test)]
@@ -299,6 +322,11 @@ mod tests {
               },
               "links": {
                 "self": "/api/admin/v1/compat-sessions/01FSHNB530AAPR7PEV8KNBZD5Y"
+              },
+              "meta": {
+                "page": {
+                  "cursor": "01FSHNB530AAPR7PEV8KNBZD5Y"
+                }
               }
             },
             {
@@ -318,6 +346,11 @@ mod tests {
               },
               "links": {
                 "self": "/api/admin/v1/compat-sessions/01FSHNCZP0PPF7X0EVMJNECPZW"
+              },
+              "meta": {
+                "page": {
+                  "cursor": "01FSHNCZP0PPF7X0EVMJNECPZW"
+                }
               }
             }
           ],
@@ -362,6 +395,11 @@ mod tests {
               },
               "links": {
                 "self": "/api/admin/v1/compat-sessions/01FSHNB530AAPR7PEV8KNBZD5Y"
+              },
+              "meta": {
+                "page": {
+                  "cursor": "01FSHNB530AAPR7PEV8KNBZD5Y"
+                }
               }
             }
           ],
@@ -403,6 +441,11 @@ mod tests {
               },
               "links": {
                 "self": "/api/admin/v1/compat-sessions/01FSHNB530AAPR7PEV8KNBZD5Y"
+              },
+              "meta": {
+                "page": {
+                  "cursor": "01FSHNB530AAPR7PEV8KNBZD5Y"
+                }
               }
             }
           ],
@@ -444,6 +487,11 @@ mod tests {
               },
               "links": {
                 "self": "/api/admin/v1/compat-sessions/01FSHNCZP0PPF7X0EVMJNECPZW"
+              },
+              "meta": {
+                "page": {
+                  "cursor": "01FSHNCZP0PPF7X0EVMJNECPZW"
+                }
               }
             }
           ],
@@ -454,5 +502,155 @@ mod tests {
           }
         }
         "#);
+
+        // Test count=false
+        let request = Request::get("/api/admin/v1/compat-sessions?count=false")
+            .bearer(&token)
+            .empty();
+        let response = state.request(request).await;
+        response.assert_status(StatusCode::OK);
+        let body: serde_json::Value = response.json();
+        assert_json_snapshot!(body, @r#"
+        {
+          "data": [
+            {
+              "type": "compat-session",
+              "id": "01FSHNB530AAPR7PEV8KNBZD5Y",
+              "attributes": {
+                "user_id": "01FSHN9AG0MZAA6S4AF7CTV32E",
+                "device_id": "LoieH5Iecx",
+                "user_session_id": null,
+                "redirect_uri": null,
+                "created_at": "2022-01-16T14:41:00Z",
+                "user_agent": null,
+                "last_active_at": null,
+                "last_active_ip": null,
+                "finished_at": null,
+                "human_name": null
+              },
+              "links": {
+                "self": "/api/admin/v1/compat-sessions/01FSHNB530AAPR7PEV8KNBZD5Y"
+              },
+              "meta": {
+                "page": {
+                  "cursor": "01FSHNB530AAPR7PEV8KNBZD5Y"
+                }
+              }
+            },
+            {
+              "type": "compat-session",
+              "id": "01FSHNCZP0PPF7X0EVMJNECPZW",
+              "attributes": {
+                "user_id": "01FSHNB530AJ6AC5HQ9X6H4RP4",
+                "device_id": "ZXyvelQWW9",
+                "user_session_id": null,
+                "redirect_uri": null,
+                "created_at": "2022-01-16T14:42:00Z",
+                "user_agent": null,
+                "last_active_at": null,
+                "last_active_ip": null,
+                "finished_at": "2022-01-16T14:43:00Z",
+                "human_name": null
+              },
+              "links": {
+                "self": "/api/admin/v1/compat-sessions/01FSHNCZP0PPF7X0EVMJNECPZW"
+              },
+              "meta": {
+                "page": {
+                  "cursor": "01FSHNCZP0PPF7X0EVMJNECPZW"
+                }
+              }
+            }
+          ],
+          "links": {
+            "self": "/api/admin/v1/compat-sessions?count=false&page[first]=10",
+            "first": "/api/admin/v1/compat-sessions?count=false&page[first]=10",
+            "last": "/api/admin/v1/compat-sessions?count=false&page[last]=10"
+          }
+        }
+        "#);
+
+        // Test count=only
+        let request = Request::get("/api/admin/v1/compat-sessions?count=only")
+            .bearer(&token)
+            .empty();
+        let response = state.request(request).await;
+        response.assert_status(StatusCode::OK);
+        let body: serde_json::Value = response.json();
+        assert_json_snapshot!(body, @r#"
+        {
+          "meta": {
+            "count": 2
+          },
+          "links": {
+            "self": "/api/admin/v1/compat-sessions?count=only"
+          }
+        }
+        "#);
+
+        // Test count=false with filtering
+        let request = Request::get(format!(
+            "/api/admin/v1/compat-sessions?count=false&filter[user]={}",
+            alice.id
+        ))
+        .bearer(&token)
+        .empty();
+        let response = state.request(request).await;
+        response.assert_status(StatusCode::OK);
+        let body: serde_json::Value = response.json();
+        assert_json_snapshot!(body, @r#"
+        {
+          "data": [
+            {
+              "type": "compat-session",
+              "id": "01FSHNB530AAPR7PEV8KNBZD5Y",
+              "attributes": {
+                "user_id": "01FSHN9AG0MZAA6S4AF7CTV32E",
+                "device_id": "LoieH5Iecx",
+                "user_session_id": null,
+                "redirect_uri": null,
+                "created_at": "2022-01-16T14:41:00Z",
+                "user_agent": null,
+                "last_active_at": null,
+                "last_active_ip": null,
+                "finished_at": null,
+                "human_name": null
+              },
+              "links": {
+                "self": "/api/admin/v1/compat-sessions/01FSHNB530AAPR7PEV8KNBZD5Y"
+              },
+              "meta": {
+                "page": {
+                  "cursor": "01FSHNB530AAPR7PEV8KNBZD5Y"
+                }
+              }
+            }
+          ],
+          "links": {
+            "self": "/api/admin/v1/compat-sessions?filter[user]=01FSHN9AG0MZAA6S4AF7CTV32E&count=false&page[first]=10",
+            "first": "/api/admin/v1/compat-sessions?filter[user]=01FSHN9AG0MZAA6S4AF7CTV32E&count=false&page[first]=10",
+            "last": "/api/admin/v1/compat-sessions?filter[user]=01FSHN9AG0MZAA6S4AF7CTV32E&count=false&page[last]=10"
+          }
+        }
+        "#);
+
+        // Test count=only with filtering
+        let request =
+            Request::get("/api/admin/v1/compat-sessions?count=only&filter[status]=active")
+                .bearer(&token)
+                .empty();
+        let response = state.request(request).await;
+        response.assert_status(StatusCode::OK);
+        let body: serde_json::Value = response.json();
+        assert_json_snapshot!(body, @r#"
+        {
+          "meta": {
+            "count": 1
+          },
+          "links": {
+            "self": "/api/admin/v1/compat-sessions?filter[status]=active&count=only"
+          }
+        }
+        "#);
     }
 }
